@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2012, the codeeditor development team
+# Copyright (C) 2013, the codeeditor development team
 #
 # IEP is distributed under the terms of the (new) BSD License.
 # The full license can be found in 'license.txt'.
@@ -126,14 +126,6 @@ class IndentationGuides(object):
         doc = self.document()
         viewport = self.viewport()
         
-        # Get which part to paint. Just do all to avoid glitches
-        w = self.getLineNumberAreaWidth()
-        y1, y2 = 0, self.height()
-        #y1, y2 = event.rect().top()-10, event.rect().bottom()+10
-        
-        # Get cursor
-        cursor = self.cursorForPosition(QtCore.QPoint(0,y1))
-        
         # Get multiplication factor and indent width
         indentWidth = self.indentWidth()
         if self.indentUsingSpaces():
@@ -152,31 +144,25 @@ class IndentationGuides(object):
         painter.setPen(pen)
         offset = doc.documentMargin() + self.contentOffset().x()
         
-        #Repainting always starts at the first block in the viewport,
-        #regardless of the event.rect().y(). Just to keep it simple
-        while True:
+        def paintIndentationGuides(cursor):
             y3=self.cursorRect(cursor).top()
             y4=self.cursorRect(cursor).bottom()            
             
             bd = cursor.block().userData()            
-            if bd and bd.indentation:
+            if bd and hasattr(bd, 'indentation') and bd.indentation:
                 for x in range(indentWidth, bd.indentation * factor, indentWidth):
                     w = self.fontMetrics().width('i'*x) + offset
                     w += 1 # Put it more under the block
                     if w > 0: # if scrolled horizontally it can become < 0
                         painter.drawLine(QtCore.QLine(w, y3, w, y4))
  
-            if y4>y2:
-                break #Reached end of the repaint area
-            if not cursor.block().next().isValid():
-                break #Reached end of the text
-            
-            cursor.movePosition(cursor.NextBlock)
+        self.doForVisibleBlocks(paintIndentationGuides)
         
         # Done
         painter.end()
 
 
+
 class FullUnderlines(object):
     
     def paintEvent(self,event):
@@ -188,45 +174,71 @@ class FullUnderlines(object):
         
         """ 
         super(FullUnderlines, self).paintEvent(event)
-    
-        # Get doc and viewport
-        doc = self.document()
-        viewport = self.viewport()
-        
-        # Init painter
+   
         painter = QtGui.QPainter()
-        painter.begin(viewport)
-        
-        # Prepare
-        y1, y2 = 0, self.height()
-        cursor = self.cursorForPosition(QtCore.QPoint(0,y1))
+        painter.begin(self.viewport())
+ 
         margin = self.document().documentMargin()
         w = self.viewport().width()
-        
-        
-        while True:
+   
+        def paintUnderline(cursor):
             y = self.cursorRect(cursor).bottom() 
-            
+
             bd = cursor.block().userData()            
-            if bd and bd.fullUnderlineFormat is not None:
-                # Apply pen
-                pen = QtGui.QPen(bd.fullUnderlineFormat.fore)
-                pen.setStyle(bd.fullUnderlineFormat.linestyle)
-                painter.setPen(pen)
-                # Paint
-                painter.drawLine(QtCore.QLine(margin, y, w - 2*margin, y))
-            
-            # Next block (or not)
-            if y > y2:
-                break #Reached end of the repaint area
-            if not cursor.block().next().isValid():
-                break #Reached end of the text
-            cursor.movePosition(cursor.NextBlock)
+            try:
+                fullUnderlineFormat = bd.fullUnderlineFormat
+            except AttributeError:
+                pass  # fullUnderlineFormat may not be an attribute
+            else:
+                if fullUnderlineFormat is not None:
+                    # Apply pen
+                    pen = QtGui.QPen(fullUnderlineFormat.fore)
+                    pen.setStyle(fullUnderlineFormat.linestyle)
+                    painter.setPen(pen)
+                    # Paint
+                    painter.drawLine(QtCore.QLine(margin, y, w - 2*margin, y))            
+        
+        self.doForVisibleBlocks(paintUnderline)
         
-        # Done
         painter.end()
 
 
+
+class CodeFolding(object):
+    def paintEvent(self,event):
+        """ paintEvent(event)
+        
+        """ 
+        super(CodeFolding, self).paintEvent(event)
+   
+        return # Code folding code is not yet complete
+   
+        painter = QtGui.QPainter()
+        painter.begin(self.viewport())
+ 
+        margin = self.document().documentMargin()
+        w = self.viewport().width()
+   
+   
+        def paintCodeFolders(cursor):
+            y = self.cursorRect(cursor).top() 
+            h = self.cursorRect(cursor).height()
+            rect = QtCore.QRect(margin, y, h, h)
+            text = cursor.block().text()           
+            if text.rstrip().endswith(':'):
+                painter.drawRect(rect)
+                painter.drawText(rect, QtCore.Qt.AlignVCenter | QtCore.Qt.AlignHCenter, "-")
+                # Apply pen
+                
+                # Paint
+                #painter.drawLine(QtCore.QLine(margin, y, w - 2*margin, y))            
+            
+        self.doForVisibleBlocks(paintCodeFolders)
+        
+        painter.end()
+        
+
+
 class LongLineIndicator(object):
     
     # Register style element
@@ -295,6 +307,8 @@ class LongLineIndicator(object):
         super(LongLineIndicator, self).paintEvent(event)
 
 
+
+
 class ShowWhitespace(object):
     
     def showWhitespace(self):
@@ -330,6 +344,8 @@ class ShowLineEndings(object):
             option.setFlags(option.flags() & ~option.ShowLineAndParagraphSeparators)
         self.document().setDefaultTextOption(option)
 
+
+
 class LineNumbers(object):
     
     # Margin on both side of the line numbers
@@ -455,8 +471,7 @@ class LineNumbers(object):
                 if blockNumber == currentBlockNumber:
                     painter.setFont(font2)
                 
-                painter.drawText(0, y-offset,
-                    editor.getLineNumberAreaWidth()-margin, 50,
+                painter.drawText(0, y-offset, w-margin, 50,
                     Qt.AlignRight, str(blockNumber+1))
                 
                 # Set font back
@@ -527,7 +542,8 @@ class LineNumbers(object):
         self.blockCountChanged.connect(self.__onBlockCountChanged)
         self.fontChanged.connect(self.__onBlockCountChanged)
         self.__onBlockCountChanged()
-        
+        self.addLeftMargin(LineNumbers, self.getLineNumberAreaWidth)
+    
     
     def gotoLinePopup(self):
         """ Popup the little widget to quickly goto a certain line.
@@ -548,8 +564,8 @@ class LineNumbers(object):
                 self.__onBlockCountChanged()
                 self.__lineNumberArea.show()
             else:
-                self.setViewportMargins(0,0,0,0)
                 self.__lineNumberArea.hide()
+            self.updateMargins()
     
     
     def getLineNumberAreaWidth(self):
@@ -562,30 +578,228 @@ class LineNumbers(object):
         lastLineNumber = self.blockCount() 
         margin = self._LineNumberAreaMargin
         return self.fontMetrics().width(str(lastLineNumber)) + 2*margin
-        
+    
+    
     def __onBlockCountChanged(self,count = None):
         """
         Update the line number area width. This requires to set the 
         viewport margins, so there is space to draw the linenumber area
         """
         if self.__showLineNumbers:
-            self.setViewportMargins(self.getLineNumberAreaWidth(),0,0,0)
+            self.updateMargins()
+    
     
     def resizeEvent(self,event):
         super(LineNumbers, self).resizeEvent(event)
         
         #On resize, resize the lineNumberArea, too
         rect=self.contentsRect()
-
-        self.__lineNumberArea.setGeometry(rect.x(),rect.y(),
-            self.getLineNumberAreaWidth(),rect.height())
-            
+        m = self.getLeftMargin(LineNumbers)
+        w = self.getLineNumberAreaWidth()
+        self.__lineNumberArea.setGeometry(  rect.x()+m, rect.y(),
+                                            w, rect.height())
+    
     def paintEvent(self,event):
         super(LineNumbers, self).paintEvent(event)
         #On repaint, update the complete line number area
-        self.__lineNumberArea.update(0, 0, 
-                self.getLineNumberAreaWidth(), self.height() )
+        w = self.getLineNumberAreaWidth()
+        self.__lineNumberArea.update(0, 0, w, self.height() )
+
+
+
+class BreakPoints(object):
     
+    _breakPointWidth = 11  # With of total bar, actual points are smaller
+    
+    # Register style element
+    _styleElements = [  (   'Editor.BreakPoints',
+                            'The fore- and background-color of the breakpoints.',
+                            'fore:#F66,back:#EEE', 
+                        ) ]
+    
+    class __BreakPointArea(QtGui.QWidget):
+        """ This is the widget reponsible for drawing the break points.
+        """
+        
+        def __init__(self, codeEditor):
+            QtGui.QWidget.__init__(self, codeEditor)
+            self.setCursor(QtCore.Qt.PointingHandCursor)
+        
+        def _getY(self, pos):
+            tmp = self.mapToGlobal(pos)
+            return self.parent().viewport().mapFromGlobal(tmp).y()
+        
+        def mousePressEvent(self, event):
+            self._toggleBreakPoint( self._getY(event.pos()))
+        
+        def _toggleBreakPoint(self, y):
+            # Get breakpoint corresponding to pressed pos
+            editor = self.parent()
+            cursor = editor.textCursor()
+            c1 = editor.cursorForPosition(QtCore.QPoint(0,y))
+            linenr = c1.blockNumber() + 1
+            
+            # Toggle
+            bps = self.parent()._breakPoints
+            if linenr in bps:
+                bps.discard(linenr)
+            else:
+                bps.add(linenr)
+            
+            # Emit signal
+            editor.breakPointsChanged.emit(editor)
+            self.update()
+        
+        def paintEvent(self, event):
+            editor = self.parent()
+            
+            if not editor.showBreakPoints():
+                return
+            
+            # Get doc and viewport
+            doc = editor.document()
+            viewport = editor.viewport()
+            
+            # Get format and margin
+            format = editor.getStyleElementFormat('editor.breakpoints')
+            margin = 1
+            w = editor._breakPointWidth
+            bulletWidth = w - 2*margin
+            
+            # Init painter
+            painter = QtGui.QPainter()
+            painter.begin(self)
+            
+            # Get which part to paint. Just do all to avoid glitches
+            y1, y2 = 0, editor.height()
+            
+            # Get offset        
+            tmp = self.mapToGlobal(QtCore.QPoint(0,0))
+            offset = viewport.mapFromGlobal(tmp).y()
+            
+            #Draw the background        
+            painter.fillRect(QtCore.QRect(0, y1, w, y2), format.back)
+            
+            # Get debug indicator and list of sorted breakpoints
+            debugBlockIndicator = editor._debugLineIndicator-1
+            blocknumbers = [i-1 for i in sorted(self.parent()._breakPoints)]
+            if not (blocknumbers or debugBlockIndicator > 0):
+                return
+            if not blocknumbers:  blocknumbers.append(-1)  # Safes a test below
+            
+            # Get cursor
+            cursor = editor.cursorForPosition(QtCore.QPoint(0,y1))
+            
+            # Init painter with font and color
+            painter.setPen(QtGui.QColor('#777'))
+            painter.setBrush(format.fore)
+            painter.setRenderHint(painter.Antialiasing)
+            
+            
+            #Repainting always starts at the first block in the viewport,
+            #regardless of the event.rect().y(). Just to keep it simple
+            while True:
+                blockNumber = cursor.block().blockNumber()
+                
+                # Done?
+                if blockNumber > blocknumbers[-1] and blockNumber > debugBlockIndicator:
+                    break
+                if not cursor.block().next().isValid():
+                    break #Reached end of the text
+                
+                # Draw
+                if blockNumber in blocknumbers:
+                    y = editor.cursorRect(cursor).center().y()
+                    y -= bulletWidth * 0.5
+                    painter.drawEllipse(margin, y, bulletWidth, bulletWidth)
+                if blockNumber == debugBlockIndicator:
+                    y = editor.cursorRect(cursor).center().y()
+                    y -= bulletWidth * 0.25
+                    painter.setBrush(QtGui.QColor('#6F6'))
+                    #painter.drawRect(margin, y, bulletWidth, 0.5*bulletWidth)
+                    painter.drawEllipse(margin, y, bulletWidth, 0.5*bulletWidth)
+                    painter.setBrush(format.fore)
+                
+                cursor.movePosition(cursor.NextBlock)
+            
+            # Done
+            painter.end()
+    
+    
+    def __init__(self, *args, **kwds):
+        self.__breakPointArea = None
+        super(BreakPoints, self).__init__(*args, **kwds)
+        # Create widget that draws the breakpoints
+        self.__breakPointArea = self.__BreakPointArea(self)
+        self.addLeftMargin(BreakPoints, self.getBreakPointAreaWidth)
+        self._breakPoints = set()
+        self._debugLineIndicator = 0
+    
+    
+    def breakPoints(self):
+        """ A list of breakpoints for this editor.
+        """
+        return list(sorted(self._breakPoints))
+    
+    
+    def clearBreakPoints(self):
+        """ Remove all breakpoints for this editor.
+        """
+        self._breakPoints = set()
+        self.breakPointsChanged.emit(self)
+    
+    
+    def setDebugLineIndicator(self, linenr):
+        """ Set the debug line indicator to the given line number.
+        If None or 0, the indicator is hidden.
+        """
+        linenr = int(linenr or 0)
+        if linenr != self._debugLineIndicator:
+            self._debugLineIndicator = linenr
+            self.__breakPointArea.update()
+    
+    
+    def getBreakPointAreaWidth(self):
+        if not self.__showBreakPoints:
+            return 0
+        else:
+            return self._breakPointWidth
+    
+    
+    def showBreakPoints(self):
+        return self.__showBreakPoints
+    
+    @ce_option(True)
+    def setShowBreakPoints(self, value):
+        self.__showBreakPoints = bool(value)
+        # Note that this method is called before the __init__ is finished,
+        # so that the area is not yet created.
+        if self.__breakPointArea:
+            if self.__showBreakPoints:
+                self.__breakPointArea.show()
+            else:
+                self.__breakPointArea.hide()
+                self.clearBreakPoints()
+            self.updateMargins()
+    
+    
+    def resizeEvent(self,event):
+        super(BreakPoints, self).resizeEvent(event)
+        
+        #On resize, resize the breakpointArea, too
+        rect=self.contentsRect()
+        m = self.getLeftMargin(BreakPoints)
+        w = self.getBreakPointAreaWidth()
+        self.__breakPointArea.setGeometry(  rect.x()+m, rect.y(),
+                                            w, rect.height())
+    
+    def paintEvent(self,event):
+        super(BreakPoints, self).paintEvent(event)
+        #On repaint, update the complete breakPointArea
+        w = self.getBreakPointAreaWidth()
+        self.__breakPointArea.update(0, 0, w, self.height() )
+
+
 
 class Wrap(object):
     
