@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2012, the IEP development team
+# Copyright (C) 2013, the IEP development team
 #
 # IEP is distributed under the terms of the (new) BSD License.
 # The full license can be found in 'license.txt'.
@@ -19,7 +19,7 @@ from pyzolib import ssdf
 from iep.iepcore.iepLogging import print
 import iep.codeeditor.parsers.tokens as Tokens
 
-from iep.codeeditor.qt import QtCore, QtGui
+from pyzolib.qt import QtCore, QtGui
 qt = QtGui
 
 
@@ -205,7 +205,8 @@ class BaseTextCtrl(codeeditor.CodeEditor):
     def __init__(self, *args, **kwds):
         super().__init__(*args, **kwds)
         
-        # Set zooming
+        # Set font and zooming
+        self.setFont(iep.config.view.fontname)
         self.setZoom(iep.config.view.zoom)
         
         # Create timer for autocompletion delay
@@ -223,10 +224,28 @@ class BaseTextCtrl(codeeditor.CodeEditor):
         
         # The string with names given to SCI_AUTOCSHOW
         self._autoCompNameString = ''
-    
+        
+        # Set autocomp accept key to default if necessary.
+        # We force it to be string (see issue 134)
+        if not isinstance(iep.config.settings.autoComplete_acceptKeys, str):
+            iep.config.settings.autoComplete_acceptKeys = 'Tab'
+        
+        # Set autocomp accept keys
+        qtKeys = []
+        for key in iep.config.settings.autoComplete_acceptKeys.split(' '):
+            if len(key) > 1:
+                key = 'Key_' + key[0].upper() + key[1:].lower()
+                qtkey = getattr(QtCore.Qt, key, None)
+            else:
+                qtkey = ord(key)
+            if qtkey:
+                qtKeys.append(qtkey)
+        self.setAutoCompletionAcceptKeys(*qtKeys)
+        
         self.completer().highlighted.connect(self.updateHelp)
         self.setIndentUsingSpaces(iep.config.settings.defaultIndentUsingSpaces)
-        self.setIndentWidth(iep.config.settings.defaultIndentWidth)
+        self.setIndentWidth(iep.config.settings.defaultIndentWidth) 
+        self.setAutocompletPopupSize(*iep.config.view.autoComplete_popupSize) 
     
     
     def _isValidPython(self):
@@ -298,9 +317,9 @@ class BaseTextCtrl(codeeditor.CodeEditor):
         by the timer. It parses the line and calls the specific methods
         to process the callTip and autoComp.
         """ 
-  
+        
         tokens = self._delayTimer._tokensUptoCursor
-           
+        
         if iep.config.settings.autoCallTip:
             # Parse the line, to get the name of the function we should calltip
             # if the name is empty/None, we should not show a signature
@@ -382,7 +401,11 @@ class BaseTextCtrl(codeeditor.CodeEditor):
         
         if self._autoCompBuffer_name:
             name = self._autoCompBuffer_name + '.' + name
-            
+        elif not self.completer().completionPrefix():
+            # Dont update help if there is no dot or prefix; 
+            # the choice would be arbitrary
+            return
+        
         # Apply
         self.processHelp(name,True)
    
