@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2012, the IEP development team
+# Copyright (C) 2013, the IEP development team
 #
 # IEP is distributed under the terms of the (new) BSD License.
 # The full license can be found in 'license.txt'.
@@ -15,7 +15,7 @@ It also has a find/replace widget that is at the bottom of the editor.
 """
 
 import os, sys, time, gc
-from iep.codeeditor.qt import QtCore, QtGui
+from pyzolib.qt import QtCore, QtGui
 
 import iep
 from iep.iepcore.compactTabWidget import CompactTabWidget
@@ -23,6 +23,7 @@ from iep.iepcore.editor import createEditor
 from iep.iepcore.baseTextCtrl import normalizePath
 from iep.iepcore.iepLogging import print
 from iep.iepcore.icons import EditorTabToolButton
+from iep import translate
 
 # Constants for the alignments of tabs
 MIN_NAME_WIDTH = 50
@@ -171,7 +172,7 @@ class FindReplaceWidget(QtGui.QFrame):
             
             # Add button
             self._hidebut.setFont( QtGui.QFont('helvetica',7) )
-            self._hidebut.setToolTip("Hide search widget (Escape)")
+            self._hidebut.setToolTip(translate('search', 'Hide search widget (Escape)'))
             self._hidebut.setIcon( iep.icons.cancel )
             self._hidebut.setIconSize(QtCore.QSize(16,16))
             vsubLayout.addWidget(self._hidebut, 0)
@@ -190,15 +191,15 @@ class FindReplaceWidget(QtGui.QFrame):
             layout.addLayout(vsubLayout, 0)
             
             # Add find text
-            self._findText.setToolTip("Find pattern")
+            self._findText.setToolTip(translate('search', 'Find pattern'))
             vsubLayout.addWidget(self._findText, 0)
             
             vsubLayout.addLayout(hsubLayout)
             
             # Add previous button
             self._findPrev = QtGui.QToolButton(self) 
-            self._findPrev.setText('Previous')
-            self._findPrev.setToolTip("Find previous occurance of the pattern")
+            t = translate('search', 'Previous ::: Find previous occurrence of the pattern.')
+            self._findPrev.setText(t);  self._findPrev.setToolTip(t.tt)
             
             hsubLayout.addWidget(self._findPrev, 0)
             
@@ -206,8 +207,8 @@ class FindReplaceWidget(QtGui.QFrame):
             
             # Add next button
             self._findNext = QtGui.QToolButton(self)
-            self._findNext.setText('Next')
-            self._findNext.setToolTip("Find next occurance of the pattern")
+            t = translate('search', 'Next ::: Find next occurrence of the pattern.')
+            self._findNext.setText(t);  self._findNext.setToolTip(t.tt)
             #self._findNext.setDefault(True) # Not possible with tool buttons
             hsubLayout.addWidget(self._findNext, 0)
         
@@ -222,24 +223,24 @@ class FindReplaceWidget(QtGui.QFrame):
             hsubLayout.setSpacing(0)
             layout.addLayout(vsubLayout, 0)
             
-            # Add replace text
-            self._replaceText.setToolTip("Replace pattern")
+            # Add replace text        
+            self._replaceText.setToolTip(translate('search', 'Replace pattern'))
             vsubLayout.addWidget(self._replaceText, 0)
             
             vsubLayout.addLayout(hsubLayout)
             
             # Add replace-all button
             self._replaceAll = QtGui.QToolButton(self) 
-            self._replaceAll.setText("Repl. all")
-            self._replaceAll.setToolTip("Replace all matches in current document")
+            t = translate('search', 'Repl. all ::: Replace all matches in current document.')
+            self._replaceAll.setText(t);  self._replaceAll.setToolTip(t.tt)
             hsubLayout.addWidget(self._replaceAll, 0)
             
             hsubLayout.addStretch(1)
             
             # Add replace button
             self._replace = QtGui.QToolButton(self)
-            self._replace.setText("Replace")
-            self._replace.setToolTip("Replace this match")
+            t = translate('search', 'Replace ::: Replace this match.')
+            self._replace.setText(t);  self._replace.setToolTip(t.tt)
             hsubLayout.addWidget(self._replace, 0)
         
         
@@ -253,13 +254,15 @@ class FindReplaceWidget(QtGui.QFrame):
             layout.addLayout(vsubLayout, 0)
             
             # Add match-case checkbox
-            self._caseCheck = QtGui.QCheckBox("Match case", self)
-            self._caseCheck.setToolTip("Find words that match case")
+            t = translate('search', 'Match case ::: Find words that match case.')
+            self._caseCheck = QtGui.QCheckBox(t, self)
+            self._caseCheck.setToolTip(t.tt)
             vsubLayout.addWidget(self._caseCheck, 0)
             
             # Add regexp checkbox
-            self._regExp = QtGui.QCheckBox("RegExp", self)
-            self._regExp.setToolTip("Find using regular expressions")
+            t = translate('search', 'RegExp ::: Find using regular expressions.')
+            self._regExp = QtGui.QCheckBox(t, self)
+            self._regExp.setToolTip(t.tt)
             vsubLayout.addWidget(self._regExp, 0)
         
         if True:
@@ -270,14 +273,16 @@ class FindReplaceWidget(QtGui.QFrame):
             layout.addLayout(vsubLayout, 0)
             
             # Add whole-word checkbox
-            self._wholeWord = QtGui.QCheckBox("Whole words", self)
-            self._wholeWord.setToolTip("Find only whole words")
+            t = translate('search', 'Whole words ::: Find only whole words.')
+            self._wholeWord = QtGui.QCheckBox(t, self)
+            self._wholeWord.setToolTip(t.tt)
             self._wholeWord.resize(60, 16)
             vsubLayout.addWidget(self._wholeWord, 0)
             
             # Add autohide dropbox
-            self._autoHide = QtGui.QCheckBox("Auto hide", self)
-            self._autoHide.setToolTip("Hide search/replace when unused for 10s")
+            t = translate('search', 'Auto hide ::: Hide search/replace when unused for 10 s.')
+            self._autoHide = QtGui.QCheckBox(t, self)
+            self._autoHide.setToolTip(t.tt)
             self._autoHide.resize(60, 16)
             vsubLayout.addWidget(self._autoHide, 0)
         
@@ -545,15 +550,20 @@ class FindReplaceWidget(QtGui.QFrame):
         # get current position
         originalPosition = editor.textCursor()
         
-        # move to beginning of text and replace all
+        # Move to beginning of text and replace all
+        # Make this a single undo operation
         cursor = editor.textCursor()
-        cursor.movePosition(cursor.Start)
-        editor.setTextCursor(cursor)
-        while self.replaceOne(wrapAround=False):
-            pass
+        cursor.beginEditBlock()
+        try:
+            cursor.movePosition(cursor.Start)
+            editor.setTextCursor(cursor)
+            while self.replaceOne(wrapAround=False):
+                pass
+        finally:
+            cursor.endEditBlock()
         
         # reset position
-        editor.setTextCursor(cursor)
+        editor.setTextCursor(originalPosition)
 
 
 
@@ -591,7 +601,7 @@ class FileTabWidget(CompactTabWidget):
         
         # Valid index?
         if index<0 or index>=self.count():
-            iep.main.setWindowTitle('Interactive Editor for Python')
+            iep.main.setMainTitle()  # No open file
         
         # Remove current item from history
         currentItem = self.currentItem()
@@ -784,6 +794,7 @@ class FileTabWidget(CompactTabWidget):
         # Keep informed about changes
         item.editor.somethingChanged.connect(self.updateItems)
         item.editor.blockCountChanged.connect(self.updateItems)
+        item.editor.breakPointsChanged.connect(self.parent().updateBreakPoints)
         
         # Store the item at the tab
         self.tabBar().setTabData(i, item)
@@ -858,6 +869,9 @@ class EditorTabs(QtGui.QWidget):
     editors. It does the saving loading etc.
     """ 
     
+    # Signal to indicate that a breakpoint has changed, emits dict
+    breakPointsChanged = QtCore.Signal(object)
+    
     # Signal to notify that a different file was selected
     currentChanged = QtCore.Signal()
     
@@ -871,6 +885,9 @@ class EditorTabs(QtGui.QWidget):
         # keep a booking of opened directories
         self._lastpath = ''
         
+        # keep track of all breakpoints
+        self._breakPoints = {}
+        
         # create tab widget
         self._tabs = FileTabWidget(self)       
         self._tabs.tabCloseRequested.connect(self.closeFile)
@@ -924,7 +941,7 @@ class EditorTabs(QtGui.QWidget):
         # Show menu if item is available
         if index >= 0:
             p = self._tabs.tabBar().tabRect(index).bottomLeft()
-            self._menu.exec_(self._tabs.tabBar().mapToGlobal(p))
+            self._menu.popup(self._tabs.tabBar().mapToGlobal(p))
     
     
     def onCurrentChanged(self):
@@ -955,6 +972,45 @@ class EditorTabs(QtGui.QWidget):
         return tmp.__iter__()
     
     
+    def updateBreakPoints(self, editor=None):
+        # Get list of editors to update keypoints for
+        if editor is None:
+            editors = self
+            self._breakPoints = {}  # Full reset
+        else:
+            editors = [editor]
+        
+        # Update our keypoints dict
+        for editor in editors:
+            fname = editor._filename or editor._name
+            if not fname:
+                continue
+            linenumbers = editor.breakPoints()
+            if linenumbers:
+                self._breakPoints[fname] = linenumbers
+            else:
+                self._breakPoints.pop(fname, None)
+        
+        # Emit signal so shells can update the kernel
+        self.breakPointsChanged.emit(self._breakPoints)
+    
+    
+    def setDebugLineIndicator(self, filename, linenr):
+        """ Set the debug line indicator. There is one indicator
+        global to IEP, corresponding to the last shell for which we
+        received the indicator.
+        """
+        # Use normcase to compare file names. OOn Windows we can have
+        # "c:\.." and "C:\..." for instance
+        filename = os.path.normcase(filename)
+        for editor in self:
+            fname = editor._filename or editor._name
+            if os.path.normcase(fname) == filename:
+                editor.setDebugLineIndicator(linenr)
+            else:
+                editor.setDebugLineIndicator(None)
+    
+    
     ## Loading ad saving files
     
     def dragEnterEvent(self, event):
@@ -982,6 +1038,8 @@ class EditorTabs(QtGui.QWidget):
         item = FileItem(editor)
         self._tabs.addItem(item)
         self._tabs.setCurrentItem(item)
+        # set focus to new file
+        editor.setFocus()
 
         return item
     
@@ -1005,7 +1063,7 @@ class EditorTabs(QtGui.QWidget):
         filter += "Pyrex (*.pyi *.pyx *.pxd);;"
         filter += "C (*.c *.h *.cpp *.c++);;"
         #filter += "Py+Cy+C (*.py *.pyw *.pyi *.pyx *.pxd *.c *.h *.cpp);;"
-        filter += "All (*.*)"
+        filter += "All (*)"
         if True:
             filenames = QtGui.QFileDialog.getOpenFileNames(self,
                 msg, startdir, filter)
@@ -1163,11 +1221,14 @@ class EditorTabs(QtGui.QWidget):
             startdir = os.path.dirname(editor._filename)
         else:
             startdir = self._lastpath 
-            # Try the project manager to suggest a path
+            # Try the file browser or project manager to suggest a path
+            fileBrowser = iep.toolManager.getTool('iepfilebrowser')
             projectManager = iep.toolManager.getTool('iepprojectmanager')
-            if projectManager:
+            if fileBrowser:
+                startdir = fileBrowser.getDefaultSavePath()
+            if projectManager and not startdir:
                 startdir = projectManager.getDefaultSavePath()
-
+        
         if not os.path.isdir(startdir):
             startdir = ''
         
@@ -1329,8 +1390,12 @@ class EditorTabs(QtGui.QWidget):
                         break
             else:
                 self._tabs.removeTab(editor)
+        
+        # Clear any breakpoints that it may have had
+        self.updateBreakPoints()
+        
         return result
-     
+    
     def closeAllFiles(self):
         """Close all files"""
         for editor in self:
