@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2012, the IEP development team
+# Copyright (C) 2013, the IEP development team
 #
 # IEP is distributed under the terms of the (new) BSD License.
 # The full license can be found in 'license.txt'.
@@ -22,10 +22,12 @@ from codeop import CommandCompiler
 import traceback
 import keyword
 import inspect # Must be in this namespace
+import bdb
 
 import yoton
 from iepkernel import guiintegration, printDirect
 from iepkernel.magic import Magician
+from iepkernel.debug import Debugger
 
 # Init last traceback information
 sys.last_type = None
@@ -102,8 +104,9 @@ class IepInterpreter:
         # Create compiler
         self._compile = CommandCompiler()
         
-        # Instantiate magician
+        # Instantiate magician and tracer
         self.magician = Magician()
+        self.debugger = Debugger()
         
         # Define prompts
         try:
@@ -115,6 +118,11 @@ class IepInterpreter:
         except AttributeError:
             sys.ps2 = "... "
         
+        # To keep track of whether to send a new prompt, and whether more
+        # code is expected.
+        self.more = 0
+        self.newPrompt = True
+        
         # Remove "THIS" directory from the PYTHONPATH
         # to prevent unwanted imports. Same for iepkernel dir
         thisPath = os.getcwd()
@@ -123,11 +131,47 @@ class IepInterpreter:
                 sys.path.remove(p)
     
     
-    def interact(self):    
-        """ Interact! (start the mainloop)
+    def run(self):    
+        """ Run (start the mainloop)
+        
+        Here we enter the main loop, which is provided by the guiApp. 
+        This event loop calls process_commands on a regular basis. 
+        
+        We may also enter the debug intereaction loop, either from a
+        request for post-mortem debugging, or *during* execution by
+        means of a breakpoint. When in this debug-loop, the guiApp event
+        loop lays still, but the debug-loop does call process-commands
+        for user interaction. 
+        
+        When the user wants to quit, SystemExit is raised (one way or
+        another). This is detected in process_commands and the exception
+        instance is stored in self._exitException. Then the debug-loop
+        is stopped if necessary, and the guiApp is told to stop its event
+        loop.
+        
+        And that brings us back here, where we exit using in order of
+        preference: self._exitException, the exception with which the
+        event loop was exited (if any), or a new exception.
+        
         """
+        
+        # Prepare
         self._prepare()
-        self._mainloop()
+        self._exitException = None
+        
+        # Enter main
+        try:
+            self.guiApp.run(self.process_commands, self.sleeptime) 
+        except SystemExit:
+            # Set self._exitException if it is not set yet
+            type, value, tb = sys.exc_info();  del tb
+            if self._exitException is None:
+                self._exitException = value
+        
+        # Exit
+        if self._exitException is None:
+            self._exitException = SystemExit()
+        raise self._exitException
     
     
     def _prepare(self):
@@ -137,7 +181,7 @@ class IepInterpreter:
         """
         
         # Reset debug status
-        self.writestatus()
+        self.debugger.writestatus()
         
         # Get startup info (get a copy, or setting the new version wont trigger!)
         while self.context._stat_startup.recv() is None:
@@ -164,31 +208,31 @@ class IepInterpreter:
         
         
         # Integrate event loop of GUI toolkit
-        self.guiApp = None
+        self.guiApp = guiintegration.App_base()
         self.guiName = guiName = startup_info['gui'].upper()
         guiError = ''
         try:
             if guiName in ['', 'NONE']:
-                pass
+                guiName = ''
             elif guiName == 'TK':
-                self.guiApp = guiintegration.Hijacked_tk()
+                self.guiApp = guiintegration.App_tk()
             elif guiName == 'WX':
-                self.guiApp = guiintegration.Hijacked_wx()
+                self.guiApp = guiintegration.App_wx()
             elif guiName == 'PYSIDE':
-                self.guiApp = guiintegration.Hijacked_pyside()
+                self.guiApp = guiintegration.App_pyside()
             elif guiName in ['PYQT4', 'QT4']:
-                self.guiApp = guiintegration.Hijacked_pyqt4()
+                self.guiApp = guiintegration.App_pyqt4()
             elif guiName == 'FLTK':
-                self.guiApp = guiintegration.Hijacked_fltk()
+                self.guiApp = guiintegration.App_fltk()
             elif guiName == 'GTK':
-                self.guiApp = guiintegration.Hijacked_gtk()
+                self.guiApp = guiintegration.App_gtk()
             else:
                 guiError = 'Unkown gui: %s' % guiName
+                guiName = ''
         except Exception: # Catch any error
             # Get exception info (we do it using sys.exc_info() because
             # we cannot catch the exception in a version independent way.
-            type, value, tb = sys.exc_info()
-            tb = None
+            type, value, tb = sys.exc_info();  del tb
             guiError = 'Failed to integrate event loop for %s: %s' % (
                 guiName, str(value))
         
@@ -197,7 +241,7 @@ class IepInterpreter:
             iepBanner = 'This is the IEP interpreter'
         if guiError:
             iepBanner += '. ' + guiError + '\n'
-        elif self.guiApp:
+        elif guiName:
             iepBanner += ' with integrated event loop for ' 
             iepBanner += guiName + '.\n'
         else:
@@ -281,130 +325,121 @@ class IepInterpreter:
                 self._scriptToRunOnStartup = filename
     
     
-    def _mainloop(self):
-        """ The actual main loop of the interpreter.
+    def process_commands(self):
+        """ Do one iteration of processing commands (the REPL).
         """
-        
-        # Get channels as local variables
-        ctrl_command = self.context._ctrl_command
-        ctrl_code = self.context._ctrl_code
-        strm_echo = self.context._strm_echo
-        strm_prompt = self.context._strm_prompt
-        stat_interpreter = self.context._stat_interpreter
-        
-        # To keep track of whether to send a new prompt, and whether more
-        # code is expected.
-        more = 0
-        newPrompt = True
-        
-        while True:
-            try:
-                
-                # Run startup script inside the loop (only the first time)
-                # so that keyboard interrupt will work
-                if self._scriptToRunOnStartup:
-                    stat_interpreter.send('Busy') 
-                    self._scriptToRunOnStartup, tmp = None, self._scriptToRunOnStartup
-                    self.runfile(tmp)
-                
-                # Set status and prompt?
-                # Prompt is allowed to be an object with __str__ method
-                if newPrompt:
-                    newPrompt = False
-                    # Write prompt (note that the second "if" is not an "elif"!
-                    preamble = ''
-                    if self._dbFrames:
-                        preamble = '('+self._dbFrameName+')'
-                    if more:
-                        strm_prompt.send(preamble+str(sys.ps2))
-                    else:
-                        strm_prompt.send(preamble+str(sys.ps1))
-                
-                if True:
-                    # Determine state. The message is really only send
-                    # when the state is different. Note that the kernelbroker
-                    # can also set the state ("Very busy", "Busy", "Dead")
-                    if self._dbFrames:
-                        stat_interpreter.send('Debug')
-                    elif more:
-                        stat_interpreter.send('More')
-                    else:
-                        stat_interpreter.send('Ready')
-                
-                
-                # Are we still connected?
-                if sys.stdin.closed or not self.context.connection_count:
-                    # Exit from main loop
-                    break
-                
-                # Get channel to take a message from
-                ch = yoton.select_sub_channel(ctrl_command, ctrl_code)
-                
-                if ch is None:
-                    pass # No messages waiting
-                
-                elif ch is ctrl_command:
-                    # Read command 
-                    line1 = ctrl_command.recv(False) # Command
-                    if line1:
-                        # Notify what we're doing
-                        strm_echo.send(line1)
-                        stat_interpreter.send('Busy')
-                        newPrompt = True
-                        # Convert command
-                        line2 = self.magician.convert_command(line1.rstrip('\n'))
-                        # Execute actual code
-                        if line2 is not None:
-                            for line3 in line2.split('\n'): # not splitlines!
-                                more = self.pushline(line3)
-                        else:
-                            more = False
-                            self._resetbuffer()
-                
-                elif ch is ctrl_code:
-                    # Read larger block of code (dict)
-                    msg = ctrl_code.recv(False)
-                    if msg:
-                        # Notify what we're doing
-                        # (runlargecode() sends on stdin-echo)
-                        stat_interpreter.send('Busy')
-                        newPrompt = True
-                        # Execute code
-                        self.runlargecode(msg)
-                        # Reset more stuff
-                        self._resetbuffer()
-                        more = False
-                
-                else:
-                    # This should not happen, but if it does, just flush!
-                    ch.recv(False)
-                
-                # Keep GUI toolkit up to date
-                if self.guiApp:
-                    self.guiApp.processEvents()
-                
-                # Wait for a bit at each round
-                time.sleep(self.sleeptime) # 50 ms
+        try:
             
-            
-            except KeyboardInterrupt:
-                self.write("\nKeyboardInterrupt\n")
+            self._process_commands()
+        
+        except KeyboardInterrupt:
+            self.write("\nKeyboardInterrupt\n")
+            self._resetbuffer()
+            self.more = 0
+        except TypeError:
+            # For some reason, when wx is integrated, keyboard interrupts
+            # result in a TypeError.
+            # I tried to find the source, but did not find it. If anyone
+            # has an idea, please e-mail me!
+            if self.guiName == 'WX':
+                self.write("\nKeyboard Interrupt\n") # space to see difference
                 self._resetbuffer()
-                more = 0
-            except TypeError:
-                # For some reason, when wx is hijacked, keyboard interrupts
-                # result in a TypeError.
-                # I tried to find the source, but did not find it. If anyone
-                # has an idea, please e-mail me!
-                if self.guiName == 'WX':
-                    self.write("\nKeyboard Interrupt\n") # space to see difference
-                    self._resetbuffer()
-                    more = 0
-            except SystemExit:
-                # Exit from interpreter
-                return
+                self.more = 0
+        except SystemExit:
+            # Get and store the exception so we can raise it later
+            type, value, tb = sys.exc_info();  del tb
+            self._exitException = value
+            # Stop debugger if it is running
+            self.debugger.stopinteraction()
+            # Exit from interpreter. Exit in the appropriate way
+            self.guiApp.quit()  # Is sys.exit() by default
     
     
+    def _process_commands(self):
+        
+        # Run startup script inside the loop (only the first time)
+        # so that keyboard interrupt will work
+        if self._scriptToRunOnStartup:
+            self.context._stat_interpreter.send('Busy') 
+            self._scriptToRunOnStartup, tmp = None, self._scriptToRunOnStartup
+            self.runfile(tmp)
+        
+        # Set status and prompt?
+        # Prompt is allowed to be an object with __str__ method
+        if self.newPrompt:
+            self.newPrompt = False
+            # Write prompt (note that the second "if" is not an "elif"!
+            preamble = ''
+            if self._dbFrames:
+                preamble = '('+self._dbFrameName+')'
+            if self.more:
+                self.context._strm_prompt.send(preamble+str(sys.ps2))
+            else:
+                self.context._strm_prompt.send(preamble+str(sys.ps1))
+        
+        if True:
+            # Determine state. The message is really only send
+            # when the state is different. Note that the kernelbroker
+            # can also set the state ("Very busy", "Busy", "Dead")
+            if self._dbFrames:
+                self.context._stat_interpreter.send('Debug')
+            elif self.more:
+                self.context._stat_interpreter.send('More')
+            else:
+                self.context._stat_interpreter.send('Ready')
+        
+        
+        # Are we still connected?
+        if sys.stdin.closed or not self.context.connection_count:
+            # Exit from main loop.
+            # This will raise SystemExit and will shut us down in the 
+            # most appropriate way
+            sys.exit()
+        
+        # Get channel to take a message from
+        ch = yoton.select_sub_channel(self.context._ctrl_command, self.context._ctrl_code)
+        
+        if ch is None:
+            pass # No messages waiting
+        
+        elif ch is self.context._ctrl_command:
+            # Read command 
+            line1 = self.context._ctrl_command.recv(False) # Command
+            if line1:
+                # Notify what we're doing
+                self.context._strm_echo.send(line1)
+                self.context._stat_interpreter.send('Busy')
+                self.newPrompt = True
+                # Convert command
+                line2 = self.magician.convert_command(line1.rstrip('\n'))
+                # Execute actual code
+                if line2 is not None:
+                    for line3 in line2.split('\n'): # not splitlines!
+                        self.more = self.pushline(line3)
+                else:
+                    self.more = False
+                    self._resetbuffer()
+        
+        elif ch is self.context._ctrl_code:
+            # Read larger block of code (dict)
+            msg = self.context._ctrl_code.recv(False)
+            if msg:
+                # Notify what we're doing
+                # (runlargecode() sends on stdin-echo)
+                self.context._stat_interpreter.send('Busy')
+                self.newPrompt = True
+                # Execute code
+                self.runlargecode(msg)
+                # Reset more stuff
+                self._resetbuffer()
+                self.more = False
+        
+        else:
+            # This should not happen, but if it does, just flush!
+            ch.recv(False)
+
+
+    
     ## Running code in various ways
     # In all cases there is a call for compilecode and a call to execcode
     
@@ -609,11 +644,33 @@ class IepInterpreter:
         
         The globals variable is used when in debug mode.
         """
+        
+        # Apply breakpoints
+        # So breakpoints are updated at each time a command is given,
+        # including commands like "db continue".
+        try:
+            breaks = self.context._stat_breakpoints.recv()
+            if self.debugger.breaks:
+                self.debugger.clear_all_breaks()
+            if breaks:  # Can be None
+                for fname in breaks:
+                    for linenr in breaks[fname]:
+                        self.debugger.set_break(fname, linenr)
+        except Exception:
+            type, value, tb = sys.exc_info(); del tb
+            print('Error while setting breakpoints: %s' % str(value))
+        
+        
         try:
             if self._dbFrames:
                 exec(code, self.globals, self.locals)
             else:
+                # Turn debugger on at this point. If there are no breakpoints,
+                # the tracing is disabled for better performance.
+                self.debugger.set_on() 
                 exec(code, self.locals)
+        except bdb.BdbQuit:
+            print("Program execution stopped from debugger.")
         except Exception:
             time.sleep(0.2) # Give stdout some time to send data
             self.showtraceback()
@@ -624,37 +681,12 @@ class IepInterpreter:
     
     ## Writing and error handling
     
-    # todo: remove this method
+    
     def write(self, text):
         """ Write errors. """
         sys.stderr.write( text )
     
     
-    def writestatus(self):
-        """ Write the status when in ready state.
-        Writes STATE to Ready or Debug and writes DEBUG (info).
-        """
-        
-        # Collect frames info
-        frames = []
-        for f in self._dbFrames:
-            # Get fname and lineno, and correct if required
-            fname, lineno = f.f_code.co_filename, f.f_lineno
-            fname, lineno = self.correctfilenameandlineno(fname, lineno)
-            if not fname.startswith('<'):
-                fname2 = os.path.abspath(fname)
-                if os.path.isfile(fname2):
-                    fname = fname2
-            # Build string
-            text = 'File "%s", line %i, in %s' % (
-                                    fname, lineno, f.f_code.co_name)
-            frames.append(text)
-        
-        # Send info object
-        state = {'index': self._dbFrameIndex, 'frames': frames}
-        self.context._stat_debug.send(state)
-    
-    
     def showsyntaxerror(self, filename=None):
         """Display the syntax error that just occurred.
         This doesn't display a stack trace because there isn't one.        
@@ -667,8 +699,7 @@ class IepInterpreter:
         """
         
         # Get info (do not store)
-        type, value, tb = sys.exc_info()
-        tb = None
+        type, value, tb = sys.exc_info();  del tb
         
         # Work hard to stuff the correct filename in the exception
         if filename and type is SyntaxError:
